Package: epicmodel
Title: Causal Modeling in Epidemiology
Version: 0.2.1
Authors@R: 
    person("Felix", "Forster", , "forsterepi@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-3670-9244"))
Description: Create causal models for use in epidemiological studies, 
    including sufficient-component cause models as introduced by 
    Rothman (1976) <doi:10.1093/oxfordjournals.aje.a112335>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: checkmate, cli, dagitty, DiagrammeR, dplyr, DT, ggplot2,
        gtools, methods, prompter, purrr, rlang, shiny, shinyalert,
        shinyjs, shinythemes, spsUtil, stringr, tibble, tidyr
Suggests: ggdag, ggforce, ggraph, knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
Depends: R (>= 3.5.0), magrittr
LazyData: true
VignetteBuilder: knitr
URL: https://forsterepi.github.io/epicmodel/,
        https://github.com/forsterepi/epicmodel
BugReports: https://github.com/forsterepi/epicmodel/issues
NeedsCompilation: no
Packaged: 2025-12-14 14:46:00 UTC; ffors
Author: Felix Forster [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3670-9244>)
Maintainer: Felix Forster <forsterepi@gmail.com>
Repository: CRAN
Date/Publication: 2025-12-15 08:10:02 UTC
Built: R 4.6.0; ; 2026-04-29 16:45:16 UTC; unix
